@@ -1,7 +1,7 @@
 /*
  * parameter.h - parameter C API
  *
- *   Copyright (c) 2007-2022  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2007-2023  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -45,9 +45,8 @@
 #ifndef GAUCHE_PARAMETER_H
 #define GAUCHE_PARAMETER_H
 
-/* ScmPrimitiveParameter is an opaque struct (Definition is
-   in priv/parameterP.h) that implements a simple thread-local
-   storage.  It doesn't have extra features such as filter
+/* ScmPrimitiveParameter is a basic parameter object.
+   It doesn't have extra features such as filter
    procedure or hooks.  It is useful for the parameter that needs
    to be accessed from C as well.
 
@@ -64,7 +63,15 @@ SCM_CLASS_DECL(Scm_PrimitiveParameterClass);
 /* Flag value for Scm_MakePrimitiveParameter */
 enum {
     /* value may be a promise; dereference automaticlaly forces it */
-    SCM_PARAMETER_LAZY = (1UL << 0)
+    SCM_PARAMETER_LAZY = (1UL << 0),
+
+    /* We have three kind of parameters
+         Legacy - SHARED=0, THREAD=0
+         SRFI-226 - SHARED=1, THREAD=0
+         SRFI-226 thread - SHARED=0, THREAD=1
+    */
+    SCM_PARAMETER_SHARED = (1UL << 1),
+    SCM_PARAMETER_THREAD = (1UL << 2),
 };
 
 SCM_EXTERN ScmPrimitiveParameter *Scm_MakePrimitiveParameter(ScmClass *klass,
@@ -77,6 +84,7 @@ SCM_EXTERN ScmObj Scm_PrimitiveParameterRef(ScmVM *vm,
 SCM_EXTERN ScmObj Scm_PrimitiveParameterSet(ScmVM *vm,
                                             const ScmPrimitiveParameter *p,
                                             ScmObj val);
+SCM_EXTERN int Scm_PrimitiveParameterSharedP(const ScmPrimitiveParameter *p);
 
 /* A convenience function to create a new ScmPrimitiveParameter
    and bind it to NAME in MOD.  Returns a newly created primitive
@@ -86,6 +94,21 @@ SCM_EXTERN ScmPrimitiveParameter *Scm_BindPrimitiveParameter(ScmModule *mod,
                                                              ScmObj initval,
                                                              u_long flags);
 
+/*
+ * Parameterization
+ *   An object to take the parameterization (entire mapping from parameter
+ *   objects to the current dynamic values) into Scheme world.
+ */
+
+typedef struct ScmParameterizationRec ScmParameterization;
+
+SCM_CLASS_DECL(Scm_ParameterizationClass);
+#define SCM_CLASS_PARAMETERIZATION  (&Scm_ParameterizationClass)
+#define SCM_PARAMETERIZATION(obj)   ((ScmParameterization*)obj)
+#define SCM_PARAMETERIZATIONP(obj)  SCM_ISA(obj,SCM_CLASS_PARAMETERIZATION)
+
+SCM_EXTERN ScmObj Scm_CurrentParameterization();
+
 /* TRANSIENT - exposed only for the backward compatibility - will be gone by 1.0 */
 #if GAUCHE_API_VERSION < 98
 typedef struct ScmParameterLocRec {
