@@ -1,7 +1,7 @@
 ;;;
 ;;; binary.ftype - foreign types and foreign objects
 ;;;
-;;;   Copyright (c) 2011-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2011-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -59,7 +59,7 @@
   (use gauche.record)
   (use gauche.sequence)
   (use scheme.list)
-  (use srfi-13)
+  (use srfi.13)
   (use binary.io)
   (use util.match)
   (export
