@@ -1,7 +1,7 @@
 ;;;
 ;;; port.stk - stk compatibility interface
 ;;;
-;;;   Copyright (c) 2000-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -36,7 +36,7 @@
 (define-module compat.stk
   (use scheme.list)
   (use scheme.charset)
-  (use srfi-13)
+  (use srfi.13)
   (use gauche.sequence)
   (export *argc*
           copy-tree remq remv remove string->uninterned-symbol bignum?
