@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.precomp - Precompile Scheme into C data
 ;;;
-;;;   Copyright (c) 2004-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,13 +35,13 @@
 ;;; as C code.
 
 (define-module gauche.cgen.precomp
-  (use srfi-1)
-  (use srfi-13)
+  (use scheme.list)
+  (use srfi.13)
   (use gauche.cgen)
   (use gauche.cgen.stub)
   (use gauche.cgen.tmodule)
   (use gauche.vm.insn)
-  (use gauche.parameter)
+  (use gauche.vm.debug-info)
   (use gauche.sequence)
   (use gauche.experimental.lamb)
   (use file.util)
@@ -63,17 +63,17 @@
 ;;  which contains forms like define-module, use, export, etc.
 ;;
 ;;  A generated C file contains an initialization function, named
-;;  Scm_Init_<foo> by default.   Typically it is called by "extention
+;;  Scm_Init_<foo> by default.   Typically it is called by "extension
 ;;  initializer", which is invoked when the DSO file is loaded by
 ;;  dynamic-load.
 ;;
 ;;    Example1:
 ;;
-;;     *  An extention extention.so is built from extension.c,
+;;     *  An extension extension.so is built from extension.c,
 ;;        foo.scm and bar.scm.
 ;;
 ;;     *  extension.c must contain a funciton Scm_Init_extension(),
-;;        which is the extention initializer.  It is called from
+;;        which is the extension initializer.  It is called from
 ;;        (dynamic-load "extension")
 ;;
 ;;     *  By processing foo.scm and bar.scm, you'll get foo.c and
@@ -85,20 +85,20 @@
 ;;
 ;;  Sometimes sources consist of Scheme files only.  In which case,
 ;;  giving true value to ext-initializer keyword argument makes
-;;  the inialization function work as an extention initializer.
+;;  the inialization function work as an extension initializer.
 ;;
 ;;    Example2:
 ;;
-;;     * An extension extension.so is built from extention.scm.
+;;     * An extension extension.so is built from extension.scm.
 ;;
 ;;     * Processing extension.scm with :ext-initializer #t
-;;       makes generated Scm_Init_extension() work as an extention
+;;       makes generated Scm_Init_extension() work as an extension
 ;;       initializer.
 ;;
 ;;  If there are more than one Scheme files and you want to make
-;;  one of its initializer funtion as an extention initializer,
+;;  one of its initializer funtion as an extension initializer,
 ;;  give :sub-initializers argument to the 'main' source whose
-;;  initialization function becomes an extention initializer.
+;;  initialization function becomes an extension initializer.
 ;;
 ;;    Example3:
 ;;
@@ -151,6 +151,12 @@
 ;;      which isn't exported) are not included in the output.  But sometimes
 ;;      hygienic public macros expands to a call of private macros, and
 ;;      gauche.cgen.precomp cannot detect such dependencies yet.
+;;
+;; load-paths : Extra load paths prepended to the system's *load-path* during
+;;      compilation.  NB: Currnently, precompilation environment isn't isolated
+;;      enough from the host environment.  You can't load a different version
+;;      of the same library the host is using just for the precompiled file.
+;;      This option is mainly for 'include's in the source.
 
 (define (cgen-precompile src . keys)
   (with-tmodule-recording
@@ -164,9 +170,13 @@
                                :key (ext-initializer #f)
                                     ((:strip-prefix prefix) #f)
                                     ((:dso-name dso) #f)
+                                    (omit-line-directives #f)
+                                    ((:omit-debug-source-info no-source) #f)
                                     (predef-syms '())
                                     (macros-to-keep '())
                                     (single-sci-file #f)
+                                    (load-paths '())
+                                    (target-parameters '())
                                     (extra-optimization #f))
   (define (precomp-1 main src)
     (let* ([out.c (cgen-scm-path->c-file (strip-prefix src prefix))]
@@ -175,12 +185,16 @@
       (%cgen-precompile src
                         :out.c out.c
                         :dso-name (or dso (basename-sans-extension main))
+                        :omit-line-directives omit-line-directives
+                        :omit-debug-source-info no-source
                         :predef-syms predef-syms
                         :strip-prefix prefix
                         :macros-to-keep macros-to-keep
                         :extra-optimization extra-optimization
                         :ext-initializer (and (equal? src main)
                                               ext-initializer)
+                        :target-parameters target-parameters
+                        :load-paths load-paths
                         :initializer-name initname)))
   (define (tweak-sci sci) ; for consolidating sci files. returns sexp list
     (if (not sci)
@@ -217,21 +231,37 @@
 (define (%cgen-precompile src
                           :key (out.c #f)
                                (out.sci #f)
+                               (omit-line-directives #f)
+                               ((:omit-debug-source-info no-source) #f)
                                ((:strip-prefix prefix) #f)
                                (ext-initializer #f)
                                ((:dso-name dso) #f)
                                (initializer-name #f)
                                (sub-initializers '())
                                (predef-syms '())
+                               (load-paths '())
                                (macros-to-keep '())
+                               ((:target-parameters tparams) '())
                                (extra-optimization #f))
   (define (do-it)
-    (parameterize ([omitted-code '()])
+    (parameterize ([omitted-code '()]
+                   [omit-debug-source-info no-source]
+                   [target-parameters tparams])
       (setup ext-initializer sub-initializers)
       (with-input-from-file src
-        (cut emit-toplevel-executor
-             (reverse (generator-fold compile-toplevel-form '() read))))
+        (^[]
+          ;; NB: *load-path* isn't a parameter (yet), so we hack it.
+          ;; This shouldn't be the way; do not copy it.
+          (define load-path-save *load-path*)
+          (unwind-protect
+              (begin
+                (set! *load-path* (fold cons *load-path* (reverse load-paths)))
+                (emit-toplevel-executor
+                 (reverse (generator-fold compile-toplevel-form '() read))))
+            (set! *load-path* load-path-save))))
       (finalize sub-initializers)
+      (setup-debug-info)
+      (show-debug-info-stat (cgen-current-unit))
       (cgen-emit-c (cgen-current-unit))))
   (let ([out.c   (or out.c (path-swap-extension (sys-basename src) "c"))]
         [out.sci (or out.sci
@@ -247,6 +277,7 @@
                                        (basename-sans-extension out.c)))
                               initializer-name)]
                    [vm-eval-situation SCM_VM_COMPILING]
+                   [cise-emit-source-line (not omit-line-directives)]
                    [private-macros-to-keep macros-to-keep]
                    [run-extra-optimization-passes extra-optimization])
       (select-tmodule 'gauche)
@@ -255,8 +286,10 @@
              (call-with-output-file out.sci
                (^p (display ";; generated automatically.  DO NOT EDIT\n" p)
                    (display "#!no-fold-case\n" p)
-                   (parameterize ([ext-module-file p])
-                     (do-it))))]
+                   (parameterize ([ext-module-file p]
+                                  [ext-module-forms '()])
+                     (do-it)
+                     (flush-ext-module))))]
             [else
              (parameterize ([ext-module-file #f])
                (do-it))]))
@@ -299,7 +332,49 @@
 
 ;; Expr -> CompiledCode
 (define (compile-in-current-tmodule expr)
-  (compile expr (~ (current-tmodule)'module)))
+  (define mod (~ (current-tmodule)'module))
+  (cond-expand
+   ;; TRANSIENT: 0.9.12 compiler doesn't accept target parameters, so
+   ;; we hack it.
+   [gauche-0.9.12
+    (let* ([eh (get-keyword :env-header-size (target-parameters) #f)]
+           [cf (get-keyword :cont-frame-size (target-parameters) #f)])
+      (with-module gauche.internal
+        (if (or eh cf)
+          (unwind-protect
+              (begin
+                (when eh (set! ENV_HEADER_SIZE eh))
+                (when cf (set! CONT_FRAME_SIZE cf))
+                (compile expr mod))
+            (set! ENV_HEADER_SIZE %eh-orig)
+            (set! CONT_FRAME_SIZE %cf-orig))
+          (compile expr mod))))]
+   [else
+    (compile expr mod :target-params (target-parameters))]))
+
+;; TRNASIENT: !!!KLUGE!!!
+;; This is only to compile 0.9.13 by 0.9.12.  Remove this after 0.9.13 release.
+;; In 0.9.12, ENV_HEADER_SIZE and CONT_FRAME_SIZE are defined as constants,
+;; though they're still GREF-ed due to the limitation of the compiler.
+;; We want to allow to alter its value during precompilation, so we
+;; redefine them as a mutable bindings.  The GAUCHE_SUPPRESS_WARNING stuff
+;; is to suppress 'redefining constant' message.
+(with-module gauche.internal
+  (define %eh-orig ENV_HEADER_SIZE)
+  (define %cf-orig CONT_FRAME_SIZE))
+(cond-expand
+ [gauche-0.9.12
+  (define old-setting (sys-getenv "GAUCHE_SUPPRESS_WARNING"))
+  (sys-putenv "GAUCHE_SUPPRESS_WARNING" "1")
+  (with-module gauche.internal
+    (define ENV_HEADER_SIZE %eh-orig)
+    (define CONT_FRAME_SIZE %cf-orig))
+  (if old-setting
+    (sys-setenv "GAUCHE_SUPPRESS_WARNING" old-setting #t)
+    (cond-expand
+     [gauche.os.windows]                ;it doesn't have unsetenv
+     [else (sys-unsetenv "GAUCHE_SUPPRESS_WARNING")]))]
+ [else])
 
 ;;================================================================
 ;; Parameters
@@ -320,6 +395,10 @@
 ;; file, assuming the source file has standard layout.
 (define ext-module-file (make-parameter #f))
 
+;; The forms to be written out to ext-module file (sci) are pushed
+;; on this parameter.
+(define ext-module-forms (make-parameter '()))
+
 ;; list of private macros that should be included in the output.
 ;; (--keep-private-macro=name,name,...)
 ;; usually private macros (macros bound to a variable which isn't exported)
@@ -331,6 +410,12 @@
 ;; Experimental: Run extra optimization during AOT compilation.
 (define run-extra-optimization-passes (make-parameter #f))
 
+;; Compile target parameters.  See 'compile' entry in compile.scm
+(define target-parameters (make-parameter '(:cont-frame-size 7)))
+
+;; Do not emit debug-source-info with the compiled code
+(define omit-debug-source-info (make-parameter #f))
+
 ;;================================================================
 ;; Bridge to the internal stuff
 ;;
@@ -344,8 +429,6 @@
   (with-module gauche.internal vm-eval-situation))
 (define global-eq?? (with-module gauche.internal global-eq??))
 (define make-identifier (with-module gauche.internal make-identifier))
-(define pair-attributes (with-module gauche.internal pair-attributes))
-(define pair-attribute-get (with-module gauche.internal pair-attribute-get))
 
 (define-constant SCM_VM_COMPILING 2) ;; must match with vm.h
 
@@ -382,7 +465,7 @@
 
 ;; Read the first form.
 ;; We don't read the entire content of the file, since it may contain
-;; srfi-10 read-time constructor that we don't know about yet.
+;; SRFI-10 read-time constructor that we don't know about yet.
 (define (first-form src) (with-input-from-file src read))
 
 ;; Check if the first form is define-module.
@@ -426,7 +509,12 @@
 ;; code to *.sci as a transient measure and eventually we should have a way
 ;; to serialize Scheme code, including indentifier info, to the file.
 (define (write-ext-module form)
-  (cond [(ext-module-file) => (^_ (write (unwrap-syntax form) _) (newline _))]))
+  (push! (ext-module-forms) (unwrap-syntax form)))
+
+(define (flush-ext-module)
+  (cond [(ext-module-file) =>
+         (^p (dolist [f (reverse (ext-module-forms))]
+               (write f p) (newline p)))]))
 
 (define (setup ext-init? subinits)
   (cgen-decl "#include <gauche/code.h>")
@@ -494,8 +582,11 @@
 (define-global-pred =export?          export)
 (define-global-pred =export-all?      export-all)
 (define-global-pred =export-if-defined? export-if-defined)
+(define-global-pred =extend?          extend)
 (define-global-pred =provide?         provide)
 (define-global-pred =lambda?          lambda)
+(define-global-pred =include?         include)
+(define-global-pred =begin?           begin)
 
 ;; A parameter that holds the list of 'omitted' #<compiled-code> - for
 ;; example, the cliche of (CLOSURE #<compiled-code> DEFINE #<identifier> RET)
@@ -514,9 +605,13 @@
     (match form
       ;; Module related stuff
       [((? =define-module?) mod . body)
-       (write-ext-module form)
-       (with-tmodule mod
-         (fold compile-toplevel-form seed body))]
+       (receive (seed extforms)
+           (parameterize ((ext-module-forms '()))
+             (let1 seed (with-tmodule mod
+                          (fold compile-toplevel-form seed body))
+               (values seed (ext-module-forms))))
+         (write-ext-module `(define-module ,mod ,@(reverse extforms)))
+         seed)]
       [((? =select-module?) mod)
        (write-ext-module form)
        (match (dso-name)
@@ -527,25 +622,33 @@
          [_ #f])
        (select-tmodule mod)
        seed]
-      [((? =use?) mod)
-       (eval-in-current-tmodule `(use ,mod)) seed]
+      [((? =use?) . _)
+       (write-ext-module form)
+       (eval-in-current-tmodule form)
+       seed]
       [((? =export?) . syms)
        (when (list? (compile-module-exports))
          (compile-module-exports
           (lset-union eq? syms (compile-module-exports))))
        (eval-in-current-tmodule `(export ,@syms))
-       (unless (ext-module-file)
-         ;; If generate .sci file, 'export' form will be in it (as a part of
-         ;; define-module form).  Otherwise we need to do export during
-         ;; initialization.
+       ;; If generate .sci file, 'export' form will be in it.
+       ;; Otherwise we need to do export during initialization.
+       (if (ext-module-file)
+         (write-ext-module form)
          (let1 exp-specs (cgen-literal syms)
            (cgen-init
             (format "  (void)Scm_ExportSymbols(Scm_CurrentModule(), ~a);"
                     (cgen-cexpr exp-specs)))))
        seed]
-      [((? =export-all?)) (compile-module-exports #t)]
+      [((? =export-all?)) (write-ext-module form) (compile-module-exports #t)]
       [((? =export-if-defined?) . _) (write-ext-module form) seed]
       [((? =provide?) arg) (write-ext-module form) seed]
+      [((? =extend?) . _)
+       (write-ext-module form)
+       (eval-in-current-tmodule form)
+       seed]
+      [((? =begin?) . forms) (fold compile-toplevel-form seed forms)]
+      [((? =include?) . filenames) (compile-includes filenames #f seed)]
       ;; Finally, ordinary expressions.
       [else
        (let* ([compiled-code (compile-in-current-tmodule form)]
@@ -616,6 +719,30 @@
   (cgen-init (format "  Scm_VMExecuteToplevels(toplevels);"))
   )
 
+;; Handle include.  Ideally this whole routine should be shared with
+;; src/compile-1.scm.
+(define (compile-includes filenames case-fold? seed)
+  (define (do-include filename seed)
+    (unless (string? filename)
+      (error "include requires literal string, but got:" filename))
+    (with-input-from-port
+        ($ (with-module gauche.internal pass1/open-include-file)
+           filename
+           (sys-dirname (port-name (current-input-port))))
+      (^[]
+        (cond-expand
+         ;; TRANSIENT
+         [(or gauche-0.9.12 gauche-0.9.13_pre1 gauche-0.9.13_pre2 gauche-0.9.13_pre3)
+          (port-case-fold-set! (current-input-port) case-fold?)]
+         [else
+          (set! ((with-module gauche.internal port-case-fold)
+                 (current-input-port))
+                case-fold?)])
+        (unwind-protect
+            (generator-fold compile-toplevel-form seed read)
+          (close-port (current-input-port))))))
+  (fold do-include seed filenames))
+
 ;;================================================================
 ;; Special form handlers
 ;;
@@ -667,8 +794,8 @@
       ((with-module gauche.cgen.precomp handle-define-syntax) f))
     (define-macro (define-macro . f)
       ((with-module gauche.cgen.precomp handle-define-macro) f))
-    (define-macro (define-inline/syntax . f)
-      ((with-module gauche.cgen.precomp handle-define-inline/syntax) f))
+    (define-macro (define-hybrid-syntax . f)
+      ((with-module gauche.cgen.precomp handle-define-hybrid-syntax) f))
     ;; TODO - we need more general framework supporting various declarations.
     ;; for the time being, this ad-hoc solution suffice our needs.
     (define-macro (declare . f)
@@ -711,7 +838,7 @@
           (,%define ,name (,%macro ',name
                                    (,%lambda (,form ,env)
                                              (,%apply ,expr (,%cdr ,form)))
-                                   ',expr))
+                                   '((source . ,expr))))
           ((with-module gauche define-macro) ,name ,expr)))
       `((with-module gauche define-macro) ,name ,expr)))
 
@@ -731,7 +858,7 @@
       `((with-module gauche define-syntax) ,@form))
      ;; If the macro needs to exported, check if we can put it in
      ;; precompiled file (it is, if the transformer is a closure).
-     ;; Othewise, we emit the form to *.sci file.
+     ;; Otherwise, we emit the form to *.sci file.
      (when (or (symbol-exported? name)
                (memq name (private-macros-to-keep)))
        (let1 val (global-variable-ref (~ (current-tmodule)'module) name #f)
@@ -741,11 +868,11 @@
                `((with-module gauche.internal %insert-syntax-binding)
                  (current-module) ',name
                  (let ((,name ,xformer-spec)) ,name))) ; attach name to closure
-             (write-ext-module `(define-syntax . ,form))
-             #f)))]
+             (begin (write-ext-module `(define-syntax . ,form))
+                    #f))))]
     [_ (error "Malformed define-syntax" form)]))
 
-(define (handle-define-inline/syntax form)
+(define (handle-define-hybrid-syntax form)
   (match form
     [(name expr xformer-spec)
      ;; KLUDGE: We treat NAME as a macro during compiling this unit, since
@@ -762,10 +889,10 @@
              `((with-module gauche define-inline) ,name
                ((with-module gauche.internal %with-inline-transformer)
                 ,expr ,xformer-spec)))
-           (error "Currently, you can't use define-inline/syntax with \
+           (error "Currently, you can't use define-hybrid-syntax with \
                    other than er-macro-transformer in precompiled file"
                   form)))]
-    [_ (error "Malformed define-inline/syntax" form)]))
+    [_ (error "Malformed define-hybrid-syntax" form)]))
 
 (define (handle-define-constant form)
   (match form
@@ -813,6 +940,7 @@
    [code-vector-c-name :init-keyword :code-vector-c-name]
    [literals           :init-keyword :literals]
    [signature-info     :init-keyword :signature-info]
+   [debug-info-cname   :init-keyword :debug-info-cname]
    )
   (make (value)
     (let* ([code (if (run-extra-optimization-passes)
@@ -829,13 +957,14 @@
                 (cgen-safe-comment (~ code'name)))
         (format #t "            (ScmWord*)(~a), ~a,\n"
                 cvn (length cv))
-        (format #t "            ~a, ~a, ~a, ~a, SCM_NIL, ~a,\n"
+        (format #t "            ~a, ~a, ~a, ~a, ~a, ~a,\n"
                 (~ code'max-stack)
                 (~ code'required-args)
                 (~ code'optional-args)
                 (if (cgen-literal-static? code-name)
                   (cgen-cexpr code-name)
                   "SCM_FALSE")
+                "SCM_NIL"
                 (if (cgen-literal-static? signature-info)
                   (cgen-cexpr signature-info)
                   "SCM_FALSE"))
@@ -853,6 +982,8 @@
             :code-vector-c-name cvn
             :code-name code-name
             :signature-info signature-info
+            :debug-info-cname (and (not (omit-debug-source-info))
+                                   (serializable-debug-info code))
             :literals lv)))
   (init (self)
     (unless (cgen-literal-static? (~ self'code-name))
@@ -862,6 +993,9 @@
     (unless (cgen-literal-static? (~ self'signature-info))
       (print "  SCM_COMPILED_CODE("(~ self'c-name)")->signatureInfo = "
              (cgen-cexpr (~ self'signature-info)) ";"))
+    (when (~ self'debug-info-cname)
+      (print "  SCM_COMPILED_CODE("(~ self'c-name)")->debugInfo = "
+             (~ self'debug-info-cname) ";"))
     (fill-code self))
   (static (self) #t)
   )
@@ -883,6 +1017,21 @@
             (unwrap-syntax (cdr sig)))
       (unwrap-syntax sig))))
 
+;; Construct serializable debug info.
+;;   See code.c Scm_CompiledCodePushInfo for the format of debug-info.
+(define (serializable-debug-info code)
+  (and-let1 di (~ code'debug-info)
+    (let1 codevec (encode-debug-info (cgen-current-unit)
+                                     (unwrap-syntax di))
+      (record-debug-info-stat! (cgen-current-unit) codevec #f)
+      (let ([codevec-lit (cgen-literal codevec)]
+            [datum-cname (cgen-allocate-static-datum 'runtime)])
+        (cgen-init (format "  ~a = Scm_MakePackedDebugInfo(SCM_U8VECTOR(~a),\
+                                         SCM_debug_info_const_vector);"
+                           datum-cname
+                           (cgen-cexpr codevec-lit)))
+        datum-cname))))
+
 ;; Returns a list of the same length of CODE, which includes the
 ;; <cgen-literal>s corresponding to the literal values in CODE.
 ;; #f is filled in the places that don't have corresponding litaral value.
@@ -987,70 +1136,28 @@
                          il insns)])
            (cgen-literal packed)))))
 
-;;----------------------------------------------------------------
-;; <module>
-;;
-
-;; We recover modules at runtime.  Anonymous modules can be recovered
-;; if they are used as a proxy of a real module (via <tmodule>).
-(define-cgen-literal <cgen-module> <module>
-  ((name  :init-keyword :name)
-   (name-literal :init-keyword :name-literal))
-  (make (value)
-    (if-let1 n (predefined-module value)
-      (make <cgen-module> :value value
-            :c-name n
-            :name (~ value'name))
-      (let1 name (or (~ value'name)
-                     (any (^[tm] (and (eq? (~ tm'module) value) (~ tm'name)))
-                          (all-tmodules))
-                     (error "Cannot emit literal for anonymous module:" value))
-        (make <cgen-module> :value value
-              :c-name (cgen-allocate-static-datum)
-              :name name :name-literal (cgen-literal name)))))
-  (init (self)
-    (unless (predefined-module (~ self'value))
-      (format #t "  ~a = SCM_OBJ(Scm_FindModule(SCM_SYMBOL(~a), \
-                                     SCM_FIND_MODULE_CREATE)); \
-                  /* module ~a */\n"
-              (~ self'c-name)
-              (cgen-cexpr (~ self'name-literal))
-              (~ self'name))))
-  (static (self) (boolean (predefined-module (~ self'value))))
-  )
-
-(define (predefined-module module)
-  (assq-ref `((,(find-module 'null)   . "Scm_NullModule()")
-              (,(find-module 'scheme) . "Scm_SchemeModule()")
-              (,(find-module 'gauche) . "Scm_GaucheModule()")
-              (,(find-module 'gauche.internal) . "Scm_GaucheInternalModule()")
-              )
-            module))
-
-;;----------------------------------------------------------------
-;; <identifier>
-;;
-
-(define-cgen-literal <cgen-scheme-identifier> <identifier>
-  ((id-name        :init-keyword :id-name)
-   (module-literal :init-keyword :module-literal))
-  (make (value)
-    (unless ((with-module gauche.internal identifier-toplevel?) value)
-      (error "identifier with compiler environment can't be compiled" value))
-    (make <cgen-scheme-identifier> :value value
-          :c-name (cgen-allocate-static-datum)
-          :id-name (cgen-literal (unwrap-syntax value))
-          :module-literal (cgen-literal (~ value'module))))
-  (init (self)
-    (let ([name (cgen-cexpr (~ self'id-name))]
-          [cname (~ self'c-name)])
-      (format #t "  ~a = Scm_MakeIdentifier(~a, SCM_MODULE(~a), \
-                                            SCM_NIL); /* ~a#~a */\n"
-              cname name (cgen-cexpr (~ self'module-literal))
-              (cgen-safe-comment (~ self'module-literal'name))
-              (cgen-safe-comment (~ self'id-name'value)))))
-  (static (self) #f)
-  )
+;; Debug info generation.  This must be called once, after all compiled codes
+;; are processed.
+(define (setup-debug-info)
+  (when (has-unit-debug-info? (cgen-current-unit))
+    (let* ([cvec (get-debug-info-const-vector (cgen-current-unit))]
+           [vec-literal (cgen-literal (make-vector (vector-length cvec) #f))]
+           [vec-name (cgen-cexpr vec-literal)]
+           [elt-literals (map cgen-literal cvec)])
+      (cgen-decl "static ScmObj SCM_debug_info_const_vector();")
+      (cgen-body "ScmObj SCM_debug_info_const_vector()"
+                 "{"
+                 ;; This is idempotent.  No need to mutex.
+                 "  static _Bool initialized = FALSE;"
+                 "  if (!initialized) {"
+                 "    int i = 0;")
+      (dolist [e elt-literals]
+        (cgen-body #"    SCM_VECTOR_ELEMENT(~|vec-name|, i++) = \
+                           ~(cgen-cexpr e);"))
+      (cgen-body "    initialized = TRUE;"
+                 "  }"
+                 #"  return ~|vec-name|;"
+                 "}"))))
 
 ;;----------------------------------------------------------------
 ;; <macro>
