@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.unit - cgen-unit
 ;;;
-;;;   Copyright (c) 2004-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -32,8 +32,8 @@
 ;;;
 
 (define-module gauche.cgen.unit
-  (use srfi-13)
-  (use srfi-42)
+  (use srfi.13)
+  (use srfi.42)
   (use util.match)
   (use file.util)
   (use gauche.sequence)
@@ -78,6 +78,7 @@
    (transients :init-value '())  ;; transient variables
    (literals  :init-form #f)     ;; literals. see gauche.cgen.literal
    (static-data-list :init-value '()) ;; static C data, see below
+   (debug-info-bin :init-value #f)   ;; see gauche.vm.debug-info
    ))
 
 ;; API
