@@ -1,7 +1,7 @@
 ;;;
 ;;; configure.scm - configuring Gauche extensions
 ;;;
-;;;   Copyright (c) 2013-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2013-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -84,8 +84,8 @@
   (use file.util)
   (use text.tr)
   (use text.tree)
-  (use srfi-13)
-  (use srfi-113) ; sets & bags
+  (use srfi.13)
+  (use scheme.set)
   (extend gauche.config)
   (export cf-init cf-init-gauche-extension
           cf-arg-enable cf-arg-with cf-feature-ref cf-package-ref
@@ -1027,7 +1027,7 @@
                      '("FILE *f = fopen(\"conftest.out\", \"w\");\n"
                        "return ferror(f) || fclose(f) != 0;")))
 
-(define-method cf-lang-call-m ((lang <c-language>) prologue body)
+(define-method cf-lang-call-m ((lang <c-language>) prologue func-name)
   ($ cf-lang-program-m lang
      (if (equal? func-name "main")
        `(,prologue
