@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.package.util - internal utilities used in package manager
 ;;;
-;;;   Copyright (c) 2004-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -42,11 +42,45 @@
   (use file.util)
   (use file.filter)
   (use text.tr)
-  (use srfi-13)
+  (use srfi.13)
   (export run dry-run verbose-run get-password copy-templates
-          find-package-name-and-version))
+          find-package-name-and-version
+          %gosh %shell %fix-path %find-file-in-paths))
 (select-module gauche.package.util)
 
+;; for windows (Msys)
+(define (%gosh script)
+  (cond-expand
+   [gauche.os.windows #"gosh ~script"]
+   [else script]))
+
+;; for windows (Msys)
+(define (%shell command)
+  (cond-expand
+   [gauche.os.windows
+    (if-let1 sh (sys-getenv "SHELL")
+      `("cmd.exe" "/c" ,sh "-c" ,command)
+      `("cmd.exe" "/c" ,command))]
+   [else command]))
+
+;; for windows (Msys)
+(define (%fix-path path)
+  (cond-expand
+   [gauche.os.windows
+    (and path
+         (if (sys-getenv "MSYSTEM")
+           (shell-escape-string path 'posix)
+           path))]
+   [else path]))
+
+;; for windows (Msys)
+(define (%find-file-in-paths name)
+  (cond-expand
+   [gauche.os.windows
+    (find-file-in-paths name :extensions '("exe"))]
+   [else
+    (find-file-in-paths name)]))
+
 (define dry-run     (make-parameter #f))
 (define verbose-run (make-parameter #f))
 
@@ -55,7 +89,10 @@
     (print cmdline))
   (unless (dry-run)
     (let1 p (run-process (cond-expand
-                          [gauche.os.windows (shell-tokenize-string cmdline 'posix)]
+                          [gauche.os.windows
+                           (if (string? cmdline)
+                             (shell-tokenize-string cmdline 'posix)
+                             cmdline)]
                           [else `("/bin/sh" "-c" ,cmdline)])
                          :input (if stdin-string :pipe :null)
                          :wait #f)
@@ -116,7 +153,9 @@
                         :key (module-name #f)
                              (use-autoconf #f)
                              (scheme-only #f)
-                             (verbose #f))
+                             (verbose #f)
+                             (license #f)
+                             (licensor #f))
   (assume-type package-name <string>)
   (let* ([extension-name (string-tr package-name "A-Za-z_-" "a-za-z__")]
          [module-name (or module-name
@@ -124,29 +163,44 @@
          [module-path (module-name->path module-name)]
          [dst-subdir  (sys-dirname module-path)]
          [gversion    (gauche-version)]
-         [author-name (%author-name)])
+         [author-name (%author-name)]
+         [license-name (ecase license
+                         [(bsd bsd3) "BSD"]
+                         [(mit) "MIT"]
+                         [(#f) #f])])
 
     (define (filter-copy src dst executables configure-name)
       (let1 EXTENSION-NAME (string-upcase extension-name)
         (when verbose
           (format #t "Installing ~a as ~a\n" src dst))
-        (file-filter (^[in out]
-                       (port-for-each
-                        (^[line]
-                          (display
-                           (regexp-replace-all*
-                            line
-                            #/@@package@@/ package-name
-                            #/@@modname@@/ (x->string module-name)
-                            #/@@modpath@@/ (module-name->path module-name)
-                            #/@@extname@@/ extension-name
-                            #/@@EXTNAME@@/ EXTENSION-NAME
-                            #/@@configure@@/ configure-name
-                            #/@@gauche-version@@/ gversion
-                            #/@@author@@/ author-name)
-                           out)
-                          (newline out))
-                        (cut read-line in)))
+        (file-filter
+         (^[in out]
+           (port-for-each
+            (^[line]
+              (display
+               (regexp-replace-all*
+                line
+                #/@@package@@/ package-name
+                #/@@modname@@/ (x->string module-name)
+                #/@@modpath@@/ (module-name->path module-name)
+                #/@@extname@@/ extension-name
+                #/@@EXTNAME@@/ EXTENSION-NAME
+                #/@@configure@@/ configure-name
+                #/@@gauche-version@@/ gversion
+                #/@@author@@/ (if author-name
+                                (write-to-string author-name)
+                                "")
+                #/@@year@@/ ($ sys-strftime "%Y" $ sys-localtime
+                               $ sys-time)
+                #/@@licensor@@/ (or licensor
+                                    author-name
+                                    "COPYRIGHT HOLDER")
+                #/@@license@@/ (if license-name
+                                 (write-to-string license-name)
+                                 ""))
+               out)
+              (newline out))
+            (cut read-line in)))
                      :input src
                      :output dst)
         (when (member (sys-basename dst) executables)
@@ -155,20 +209,26 @@
     (make-directory* (build-path dstdir dst-subdir))
     (dolist [file (append (if use-autoconf
                             '("configure.ac")
-                            '("configure" "configure-compat"))
+                            '("configure"))
                           (if scheme-only
                             '("Makefile-pure-scheme.in"
                               "module-pure-scheme.scm")
                             '("Makefile.in" "extension.c"
                               "extension.h" "extensionlib.stub"
                               "module.scm" ))
+                          (case license
+                            [(bsd bsd3) '("COPYING--bsd3")]
+                            [(mit) '("COPYING--mit")]
+                            [(#f) '()])
                           '("package.scm" "test.scm"))]
       (let* ([src-path (build-path srcdir file)]
              [dst-name (regexp-replace*
                         file
                         #/extension/ extension-name
                         #/module/ (sys-basename module-path)
-                        #/-pure-scheme/ "")]
+                        #/-pure-scheme/ ""
+                        #/--bsd3/ ""
+                        #/--mit/ "")]
              [dst-path (if (#/^module/ file)
                          (build-path dstdir dst-subdir dst-name)
                          (build-path dstdir dst-name))])
@@ -178,14 +238,13 @@
 
 ;; Retrieve author name and email from git config, if possible
 (define (%author-name)
-  (or (and-let* ([git (find-file-in-paths "git" :extensions '("exe"))]
-                 [name  (process-output->string '(git config user.name)
-                                                :error :null
-                                                :on-abnormal-exit :ignore)]
-                 [ (and name (not (string-null? name))) ]
-                 [email (process-output->string '(git config user.email)
-                                                :error :null
-                                                :on-abnormal-exit :ignore)]
-                 [ (and email (not (string-null? email))) ])
-        (write-to-string #"~name <~|email|>"))
-      ""))
+  (and-let* ([git (%find-file-in-paths "git")]
+             [name  (process-output->string '(git config user.name)
+                                            :error :null
+                                            :on-abnormal-exit :ignore)]
+             [ (and name (not (string-null? name))) ]
+             [email (process-output->string '(git config user.email)
+                                            :error :null
+                                            :on-abnormal-exit :ignore)]
+             [ (and email (not (string-null? email))) ])
+    #"~name <~|email|>"))
