@@ -1,7 +1,7 @@
 ;;;
 ;;; parseopt.scm - yet another command-line argument parser
 ;;;
-;;;   Copyright (c) 2000-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -32,6 +32,7 @@
 ;;;
 
 (define-module gauche.parseopt
+  (use util.match)
   (export make-option-parser parse-options let-args <parseopt-error>))
 (select-module gauche.parseopt)
 
@@ -57,13 +58,21 @@
 ;;  (<option-spec> ...)
 ;; <a-spec> is (<optspec> <handler>) or
 ;; ((<optspec> <help-string>) <handler>)
+;; TRANSIENT: Better to have explanatory message for assume-type, but
+;; 0.9.12's assume-type doesn't take extra message arg.  Update after 0.9.13
+;; release.
 (define (compose-entry a-spec)
-  (let ([optspec (if (pair? (car a-spec)) (caar a-spec) (car a-spec))]
-        [helpstr (and (not (length<=? (car a-spec) 1))
-                      (cadar a-spec))]
-        [handler (cadr a-spec)])
-    (unless (string? optspec)
-      (error "option spec must be a string, but got" optspec))
+  (define (parse-spec a-spec)
+    (match a-spec
+      [((spec help-string) handler)
+       (values (assume-type spec <string>)
+               (assume-type help-string <string>)
+               handler)]
+      [(spec handler)
+       (values (assume-type spec <string>)
+               #f handler)]
+      [_ (error "Invalid command-line argument specification:" a-spec)]))
+  (receive (optspec helpstr handler) (parse-spec a-spec)
     (rxmatch-if (rxmatch #/^-*([-+\w|]+)(\*)?(?:([=:])(.+))?$/ optspec)
         (#f optnames plural? optional?  argspec)
       (map (cute make <option-spec>
@@ -77,19 +86,38 @@
            (string-split optnames #\|))
       (error "unrecognized option spec:" optspec))))
 
+(define (optspec-short? optspec) (= (string-length (~ optspec'name)) 1))
+(define (optspec-take-args? optspec) (not (null? (~ optspec'args))))
+
+;; Find an optspec that matches the given option.
+;; A special handling is needed for a single-letter option taking arguments;
+;; fof such an option, we allow its argument to be concatenated with the
+;; option itself; e.g. "I=s" spec can accept '-I arg', '-I=arg', and '-Iarg'.
+(define (find-matching-optspec option optspecs)
+  (define (optspec-long-match? optspec option)
+    (equal? (rxmatch->string #/^[^=]+/ option) (~ optspec'name)))
+  (define (optspec-short-match? optspec option)
+    (and (optspec-take-args? optspec)
+         (optspec-short? optspec)
+         (eqv? (string-ref option 0) (string-ref (~ optspec'name) 0))))
+  ;; It is imperative to search long option first, then search short options.
+  ;; If we have "long" and "l=s", "--long" matches the first one, while
+  ;; "-lfoo" matches the second one.
+  (or (find (cut optspec-long-match? <> option) optspecs)
+      (find (cut optspec-short-match? <> option) optspecs)))
+
 ;; From the args given at the command line, get a next option.
-;; Returns option string and rest args.
+;; Returns option string (potentially followed by arg) and rest args.
 (define (next-option args)
   (cond
    [(null? args) (values #f '())]
    [(string=? (car args) "--") (values #f (cdr args))]
-   [(#/^--?(\w[-+\w]*)(=)?/ (car args))
-    => (^m (values (m 1) (if (m 2) (cons (m'after) (cdr args)) (cdr args))))]
+   [(#/^--?(\w.*)/ (car args)) => (^m (values (m 1) (cdr args)))]
    [else (values #f args)]))
 
 ;; From the list of optarg spec and given command line arguments,
 ;; get a list of optargs.  Returns optargs and unconsumed args.
-(define (get-optargs optspec args)
+(define (get-optargs optspec option args)
   (define (get-number arg)
     (or (string->number arg)
         (errorf <parseopt-error> :option-name (~ optspec 'name)
@@ -115,7 +143,7 @@
                        "the argument for option ~a is not valid sexp: ~s"
                        (~ optspec 'name) arg)])
       (read-from-string arg)))
-  (define (process-args)
+  (define (process-args args)
     (let loop ([spec (~ optspec 'args)]
                [args args]
                [optargs '()])
@@ -139,24 +167,41 @@
                [else (error "unknown option argument spec:" (car spec))])])
       ))
 
-  (if (~ optspec 'arg-optional?)
-    (if (or (null? args)
-            (#/^-/ (car args)))
-      (values (make-list (length (~ optspec 'args)) #f) args)
-      (process-args))
-    (process-args))
-  )
+  (cond [(and (#/^[^=]+=/ option))
+         => (^m (let1 arg (rxmatch-after m)
+                  (process-args (cons arg args))))]
+        [(and (optspec-short? optspec)
+              (optspec-take-args? optspec)
+              (> (string-length option) 1))
+         ;; single-letter with concatenated argument
+         (process-args (cons (substring option 1 (string-length option)) args))]
+        [(~ optspec 'arg-optional?)
+         (if (or (null? args)
+                 (#/^-/ (car args)))
+           (values (make-list (length (~ optspec 'args)) #f) args)
+           (process-args args))]
+        [else
+         (process-args args)]))
 
 ;; Now, this is the argument parser body.
 (define (parse-cmdargs args speclist fallback)
   (let loop ([args args])
     (receive (option nextargs) (next-option args)
       (if option
-        (cond [(find (^e (equal? option (~ e'name))) speclist)
+        (cond [(find-matching-optspec option speclist)
                => (^[entry] (receive (optargs nextargs)
-                                (get-optargs entry nextargs)
+                                (get-optargs entry option nextargs)
                               (apply (~ entry'handler) optargs)
                               (loop nextargs)))]
+              ;; For unknown options including '=', we split at the first '='
+              ;; and take the first part as an unknown option.  This is not
+              ;; exactly the right thing; if we get "--unknown=--known" in
+              ;; command line, where 'unknown' is an unknown option and
+              ;; 'known' is a known option, this passes 'unknown' to the
+              ;; fallback handler, but keep processing 'known' as an option.
+              ;; However, we need this behavior for the backward compatibility.
+              [(#/^([^=]+)=/ option)
+               => (^m (fallback (m 1) (cons (m 'after) nextargs) loop))]
               [else (fallback option nextargs loop)])
         nextargs))))
 
