@@ -1,7 +1,7 @@
 ;;;
 ;;; selector - simple event loop by select()
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -33,7 +33,7 @@
 
 
 (define-module gauche.selector
-  (use srfi-1)
+  (use scheme.list)
   (export <selector> selector-add! selector-delete! selector-select)
   )
 (select-module gauche.selector)
