@@ -1,7 +1,7 @@
 ;;;
 ;;; math/prime.scm - utilities related to prime numbers
 ;;;
-;;;   Copyright (c) 2013-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2013-2023  Shiro Kawai  <shiro@acm.org>
 ;;;   Copyright (c) 2013  @cddddr
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
@@ -33,8 +33,8 @@
 ;;;
 
 (define-module math.prime
-  (use srfi-27)
-  (use srfi-42)
+  (use srfi.27)
+  (use srfi.42)
   (use gauche.uvector)
   (use gauche.generator)
   (use gauche.sequence)
