@@ -1,7 +1,7 @@
 ;;;
 ;;; css.scm - Generate and parse CSS
 ;;;
-;;;   Copyright (c) 2014-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2014-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -39,7 +39,7 @@
   (use util.match)
   (use text.tree)
   (use scheme.list)
-  (use srfi-13)
+  (use srfi.13)
   (export construct-css simple-selector?
           parse-css parse-css-file
           parse-css-selector-string)
